package formula;

import formula.*;
import java.util.*;

// A VarExp represents both a specific and generic cell reference. If it
// represents a generic cell reference then the row index will be -1.
class VarExp extends Exp {
  static final int NUMROWS = 100;
  static Hashtable<String, Cell[]> varTable = new Hashtable<String, Cell[]>();
  String var;
  int row;

  static void setCell(String var, int rowIndex, Exp formula) {
    Cell[] cellRow;
    Cell c;
    if (varTable.containsKey(var)) 
      cellRow = varTable.get(var);
    else {
      cellRow = new Cell[NUMROWS];
      varTable.put(var, cellRow);
    }
    if (cellRow[rowIndex] == null) 
      cellRow[rowIndex] = new Cell(formula);
    else
      cellRow[rowIndex].setCell(formula);
  }
    
  static Cell getCell(String var, int rowIndex) throws UndefinedCellException 
  {
   Cell cell = null;
    if (varTable.containsKey(var)) {
      Cell[] cellRow = varTable.get(var);
      cell = cellRow[rowIndex];
      if (cell == null) {
	throw new UndefinedCellException(var, rowIndex);
      }
    }
    else {
      throw new UndefinedCellException(var, rowIndex);
    }
    return cell;
  }

  public double eval(int rowNum) throws UndefinedCellException { 
    // if a generic reference then use the row index that has been
    // passed to eval
    int r = (row == -1 ? rowNum : row);
    Cell cell = getCell(var, r);
    try {
      return cell.eval(r);
    } catch (UndefinedCellException e) {
      e.var = var;
      throw e;
    }
  }

  public VarExp(String id, int rowNum) {
    var = id;
    row = rowNum;
  }
}
