package formula;
import java_cup.runtime.*;

%%
%{
  int getLine() { return yyline+1; }
%}
%cup
%line
NUM = [0-9]
LETTER = [a-zA-Z]
WhiteSpace = [ \t\r\f]
%%

"+" {return new Symbol(sym.ADD, yyline, 0);}
"-" {return new Symbol(sym.MINUS, yyline, 0);}
"*" {return new Symbol(sym.MULTIPLY, yyline, 0);}
"/" {return new Symbol(sym.DIVIDE, yyline, 0);}
"<" {return new Symbol(sym.LT, yyline, 0);}
">" {return new Symbol(sym.GT, yyline, 0);}
"<=" {return new Symbol(sym.LE, yyline, 0);}
">=" {return new Symbol(sym.GE, yyline, 0);}
"==" {return new Symbol(sym.EQ, yyline, 0);}
"!=" {return new Symbol(sym.NE, yyline, 0);}
"=" {return new Symbol(sym.EQUALS, yyline, 0);}
"?" {return new Symbol(sym.QMARK, yyline, 0);}
":" {return new Symbol(sym.COLON, yyline, 0);}
"," {return new Symbol(sym.COMMA, yyline, 0);}
"[" {return new Symbol(sym.LBRACKET, yyline, 0);}
"]" {return new Symbol(sym.RBRACKET, yyline, 0);}
"(" {return new Symbol(sym.LPAREN, yyline, 0);}
")" {return new Symbol(sym.RPAREN, yyline, 0);}
\n {return new Symbol(sym.NEWLINE, yyline, 0);}
"sum" {return new Symbol(sym.SUM, yyline, 0);}
"min" {return new Symbol(sym.MIN, yyline, 0);}
"max" {return new Symbol(sym.MAX, yyline, 0);}
{NUM}+|{NUM}*\.{NUM}+ {
           return new Symbol(sym.NUM, yyline, 0, new Double(yytext()));}
{LETTER}({LETTER}|{NUM}|_)* {return new Symbol(sym.ID, yyline, 0, new String(yytext()));}
{WhiteSpace} { /* ignore white space. */ }
. { System.err.printf("Illegal Character: %s at line %d\n", 
	yytext(), yyline+1);}
